# SPDX-License-Identifier: MIT

#x - horizontal motor
#y - vertical motor
#z - focus

epics_config = {
    'sample_position_x': '13IDD:m81',
    'sample_position_y': '13IDD:m83',
    'sample_position_z': '13IDD:m82',
    'sample_position_omega': '13IDD:m96'}